/* palmsched.c -- Main code for Palm Time Table, PalmOS application
   Copyright (C) 2000 Per Jonsson
   Written by Per Jonsson <poj@commsoft.se>.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.  */

/* $Id: palmsched.c,v 1.10 2000/05/26 19:24:17 poj Exp $ */

/* Changelog:
 *
 * $Log: palmsched.c,v $
 * Revision 1.10  2000/05/26 19:24:17  poj
 * Updates for teacherselect
 *
 * Revision 1.9  2000/05/26 19:06:13  poj
 * Updates for teacherselect.
 *
 * Revision 1.8  2000/05/26 18:36:15  poj
 * Bringing code up to sync. Cleaning out old stuff. Adding new stuff.
 *
 * Revision 1.7  2000/05/01 20:12:19  poj
 * *** empty log message ***
 *
 * Revision 1.6  2000/01/18 21:09:42  poj
 * Misc changes.
 *
 * Revision 1.5  2000/01/18 15:24:26  poj
 * Misc updates to make at least some things sane.
 *
 * Revision 1.4  2000/01/17 13:53:07  poj
 * Fixes, Fixes, fixes.
 * The Design of the table init-code has changed a lot.
 * It now takes less place.
 *
 * Revision 1.3  2000/01/17 12:58:34  poj
 * Initial add to CVS.
 * These files was used in a earlier version of the code.
 * The purpose of these files is to handle functions that can be shared
 * between apps.
 *
 * Revision 1.2  2000/01/16 12:41:41  poj
 * A lot has changed. In particular, the initfunction for the PSForm TTable,
 * and the format and drawing of the custom cells.
 *
 * Revision 1.1.1.1  2000/01/16 10:04:30  poj
 * Initial import into SourceForge CVS
 *
 */

#include <Pilot.h>
#include "callback.h"

#include "palmschedRsc.h"
#include "palmschedFunctions.h"
 
/* Include code to use libpojpalm */
#include "pojtable.h"
#include "pojchararray.h"

VoidHand SetUpTeacherList(int size);

/* PSFormTTableInit - Initializes TTable */
static void PSFormTTableInit ()
{
  FormPtr frm;
  TablePtr tableP;
  VoidHand list_hand;
  UInt i = 0;
  UInt j = 0;
  static char* labels[] = {"Time","Class","Teacher","Room","8:00-9:30"};
  
  frm = FrmGetActiveForm();
  
  tableP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TTable));
  list_hand = SetUpTeacherList(6);

  if (list_hand == NULL)
    { }
  
  /* Set the Header labels */
  j = 0;
  for ( i = 0; i < 7; i += 2) {
    TblSetItemStyle(tableP, 0, i, labelTableItem);
    TblSetItemPtr(tableP, 0, i, labels[j]);
    j++;
  }

  /* The Fill-ups (lines) between the true header columns */
  for ( i = 1; i < 6; i += 2) {
    TblSetItemStyle(tableP, 0, i, customTableItem);
    TblSetItemInt(tableP, 0, i, 1);
  }

  /* Fix the Line rows */
  /* Line 1 (zero-based)*/
  TblSetRowHeight(tableP, 1, 1);
  //  TblSetRowStaticHeight(tableP, 1, true);
  for ( i = 0; i < (kMaxCols - 1); i++){
    TblSetItemStyle(tableP, 1, i, customTableItem);
    TblSetItemInt(tableP, 1, i, 2); /* Horizontal line */
  }
  for ( i = 1; i < (kMaxCols - 1); i += 2)
    TblSetItemInt(tableP, 1, i, 3); /* Cross */
  
  /* Set the style of all normal cells*/
  for (i = 2; i < kMaxRows; i++) {
    /* This One sees to that the height of the row is static */
    TblSetRowStaticHeight(tableP, i, true);

    /* Fix all the cells in the row */
    for ( j = 0; j < 7; j += 2) {
      if ( j != 4) { /* All Columns Except no. 4 */
	TblSetItemStyle(tableP, i, j, textTableItem);
	TblSetItemPtr(tableP, i, j, labels[4]);
      } else { /* This is a special */
	TblSetItemStyle(tableP, i, j, customTableItem);
	TblSetItemInt(tableP, i, j, 4);
	TblSetItemPtr(tableP, i, j, list_hand);
      }
    }
  }

  /* Set the type so the line-cells become correct */
  for (i = 2; i < kMaxRows; i++) {
    for ( j = 1; j < 6; j += 2) {
      TblSetItemStyle(tableP, i, j, customTableItem);
      TblSetItemInt(tableP, i, j, 1);
    }
  }

  for (i = 0; i < kMaxCols; i++)
    TblSetColumnUsable(tableP, i, true);

  /* Line kMaxRows-1 (adds the line there)*/
  TblSetRowHeight(tableP, kMaxRows-1, 1);
  for ( i = 0; i < (kMaxCols -1); i++){
    TblSetItemStyle(tableP, kMaxRows-1, i, customTableItem);
    TblSetItemInt(tableP, kMaxRows-1, i, 2); /* Horizontal line */
  }
  for ( i = 1; i < (kMaxCols -1); i += 2)
    TblSetItemInt(tableP, kMaxRows-1, i, 3); /* Cross */

  /* Stop rows 0 and 1 from being selectable */
  TblSetRowSelectable(tableP, 0, false);
  TblSetRowSelectable(tableP, 1, false);
  TblSetRowSelectable(tableP, kMaxRows-1, false);

  /* Set the column edit indicator of on all colums */
  for ( i = 0; i < kMaxCols; i++) {
    TblSetColumnEditIndicator(tableP, i, false);
  }

  /* set the CustomDrawProcedure so we can draw our lines */
  for (i = 0; i < kMaxCols; i++) {
    TblSetCustomDrawProcedure(tableP, i, PoJTblLineDraw);
  }
  TblSetCustomDrawProcedure(tableP, 4, TeacherTblDraw);

}

/* TeCFormHandleEvent - handles events for TeCForm */
static Boolean TeCFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    
    CALLBACK_PROLOGUE

    switch (e->eType)
      {
   
      case frmOpenEvent:
	frm = FrmGetActiveForm();
	FrmDrawForm(frm);
	handled = true;
	break;

      case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
	case OKButton:
	  FrmReturnToForm(FrmGetFormId(FrmGetFormPtr(PSForm)));
	  break;
	}
	break;

      default:
	break;
      }

    CALLBACK_EPILOGUE

    return handled;
}

static Boolean PSFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    
    CALLBACK_PROLOGUE

    switch (e->eType)
      {
   
      case frmOpenEvent:
	frm = FrmGetActiveForm();
	PSFormTTableInit();
	FrmDrawForm(frm);
	handled = true;
	break;

      case menuEvent:
	MenuEraseStatus(NULL);
	
	switch(e->data.menu.itemID) {
	}
      
	handled = true;
	break;

      case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {

	}
	break;

      case tblSelectEvent: {
	UInt column = e->data.tblEnter.column;
	
	if ((column % 2) == 1) { /* If the column number is odd */
	  if (column != 0) {
	    UInt row = e->data.tblEnter.row;
	    TablePtr table = e->data.tblEnter.pTable;
	    TblMarkRowInvalid(table, row);
	    TblRedrawTable(table);
	    handled = true;
	  } 
	}else {
	  if (column == 4) {
	    SndPlaySystemSound(sndStartUp);
	    FrmPopupForm(TeCForm);
	    handled = true;
	  }
	}
	
	break; 
      }
      
      default:
	break;
      }

    CALLBACK_EPILOGUE

    return handled;
}

static Boolean ApplicationHandleEvent(EventPtr e)
{
    FormPtr frm;
    Word    formId;
    Boolean handled = false;

    if (e->eType == frmLoadEvent) {
	formId = e->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	switch(formId) {
	case PSForm:
	  FrmSetEventHandler(frm, PSFormHandleEvent);
	  break;
	case TeCForm:
	  FrmSetEventHandler(frm, TeCFormHandleEvent);
	  break;
	}
	handled = true;
    }

    return handled;
}

/* Get preferences, open (or create) app database */
static Word StartApplication(void)
{
    FrmGotoForm(PSForm);
    return 0;
}

/* Save preferences, close forms, close app database */
static void StopApplication(void)
{
    FrmSaveAllForms();
    FrmCloseAllForms();
}

/* The main event loop */
static void EventLoop(void)
{
    Word err;
    EventType e;

    do {
	EvtGetEvent(&e, evtWaitForever);
	if (! SysHandleEvent (&e))
	    if (! MenuHandleEvent (NULL, &e, &err))
		if (! ApplicationHandleEvent (&e))
		    FrmDispatchEvent (&e);
    } while (e.eType != appStopEvent);
}

/* SetUpTeacherList */
VoidHand SetUpTeacherList(int size) 
{
  VoidHand temp;
  struct poj_char3 *pList;
   
  temp = MemHandleNew(sizeof(struct poj_char3)*size);
  
  if (temp == NULL)
    return temp;

  pList = MemHandleLock(temp);
  /*
  pList[0] = "Lun";
  pList[sizeof(struct poj_char3)*1] = "Bn";
  pList[sizeof(struct poj_char3)*2] = "Bdn";
  pList[sizeof(struct poj_char3)*3] = "Bsn";
  pList[sizeof(struct poj_char3)*4] = "Bgn";
  */
  return temp;
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    Word err;

    if (cmd == sysAppLaunchCmdNormalLaunch) {

	err = StartApplication();
	if (err) return err;

	EventLoop();
	StopApplication();

    } else {
	return sysErrParamErr;
    }

    return 0;
}



  






